==> jekyll-purgecss-0.4.1.gem: lib/jekyll/hooks/purgecss.rb <==
# frozen_string_literal: true

require 'open3'
require 'digest'

class PurgecssNotFoundError < RuntimeError; end
class PurgecssRuntimeError < RuntimeError; end

Jekyll::Hooks.register(:site, :post_write, priority: 21) do |site|
  next unless Jekyll.env == 'production'

  raise PurgecssNotFoundError unless File.file?('./node_modules/.bin/purgecss')

  PURGECSS = File.join('node_modules', '.bin', 'purgecss')

  Dir.glob(File.join(site.dest, '**', '*.css')).each do |css|
    digest_filename = Digest::SHA256.hexdigest(css)
    digest_file     = File.join(site.cache_dir, digest_filename)
    purged_file     = "#{digest_file}.css"
    current_digest  = Digest::SHA256.file(css).hexdigest
    previous_digest = File.read(digest_file) if File.exist?(digest_file)

    if current_digest == previous_digest
      Jekyll.logger.info 'PurgeCSS:', "Already purged #{css}, recovering from cache"
      FileUtils.rm_f(css)
      FileUtils.cp(purged_file, css)
    else
      Jekyll.logger.info 'PurgeCSS:', "Purging #{css}"

      Open3.popen2e(
        PURGECSS,
        '--css', css,
        '--content', File.join(site.dest, '**', '*.html'), File.join(site.dest, '*.html'),
        '--out', File.dirname(css),
        chdir: site.source) do |_, o, t|

        Thread.new do
          o.each do |line|
            Jekyll.logger.info 'PurgeCSS:', line
          end
        rescue IOError => e
          Jekyll.logger.error 'PurgeCSS:', e.message
        end

        raise PurgecssRuntimeError unless t.value.success?
      end

      FileUtils.rm_f(purged_file)
      FileUtils.cp(css, purged_file)
      File.write(digest_file, current_digest)
    end
  end
end

==> jekyll-purgecss-0.5.0.gem: lib/jekyll/hooks/purgecss.rb <==
# frozen_string_literal: true

require 'open3'
require 'digest'

class PurgecssNotFoundError < RuntimeError; end
class PurgecssRuntimeError < RuntimeError; end

Jekyll::Hooks.register(:site, :post_write, priority: 21) do |site|
  next unless Jekyll.env == 'production'

  raise PurgecssNotFoundError unless File.file?('./node_modules/.bin/purgecss')

  PURGECSS = File.join('node_modules', '.bin', 'purgecss')

  allowlist   = site.config.dig('purgecss', 'allowlist')
  allowlist ||= []

  unless allowlist.is_a?(Array)
    Jekyll.logger.error 'PurgeCSS:', 'purgecss.allowlist must be array'
    allowlist = []
  end

  Dir.glob(File.join(site.dest, '**', '*.css')).each do |css|
    digest_filename = Digest::SHA256.hexdigest([css].concat(allowlist.sort).join)
    digest_file     = File.join(site.cache_dir, digest_filename)
    purged_file     = "#{digest_file}.css"
    current_digest  = Digest::SHA256.file(css).hexdigest
    previous_digest = File.read(digest_file) if File.exist?(digest_file)

    if current_digest == previous_digest
      Jekyll.logger.info 'PurgeCSS:', "Already purged #{css}, recovering from cache"
      FileUtils.rm_f(css)
      FileUtils.cp(purged_file, css)
    else
      Jekyll.logger.info 'PurgeCSS:', "Purging #{css}"

      cmd = [
        PURGECSS,
        '--css', css,
        '--content', File.join(site.dest, '**', '*.html'), File.join(site.dest, '*.html'),
        '--out', File.dirname(css)
      ]

      if allowlist.is_a? Array
        cmd << '-w'
        cmd << allowlist.join(' ')
      end

      Open3.popen2e(*cmd, chdir: site.source) do |_, o, t|
        Thread.new do
          o.each do |line|
            Jekyll.logger.info 'PurgeCSS:', line
          end
        rescue IOError => e
          Jekyll.logger.error 'PurgeCSS:', e.message
        end

        raise PurgecssRuntimeError unless t.value.success?
      end

      FileUtils.rm_f(purged_file)
      FileUtils.cp(css, purged_file)
      File.write(digest_file, current_digest)
    end
  end
end
